@@ -5,7 +5,11 @@ from dataclasses import dataclass
 from typing import Optional
 from typing import Union
 
+import click
+from chained_accounts import ChainedAccount
 from telliot_core.apps.telliot_config import TelliotConfig
+from telliot_feeds.utils.cfg import check_accounts
+from telliot_feeds.utils.cfg import setup_account
 
 
 def get_tx_explorer_url(tx_hash: str, cfg: TelliotConfig) -> str:
@@ -38,7 +42,7 @@ class NewReport:
     """NewReport event."""
 
     tx_hash: str = ""
-    eastern_time: str = ""
+    submission_timestamp: int = 0  # timestamp attached to NewReport event (NOT the time retrieved by the DVM)
     chain_id: int = 0
     link: str = ""
     query_type: str = ""
@@ -67,6 +71,26 @@ def clear_console() -> None:
         _ = os.system("clear")
 
 
+def select_account(cfg: TelliotConfig, account: str) -> Optional[ChainedAccount]:
+
+    cfg.main.chain_id = 1
+
+    if account is not None:
+        accounts = check_accounts(cfg, account)
+        click.echo(f"Your account name: {accounts[0].name if accounts else None}")
+    else:
+        account = setup_account(cfg.main.chain_id)
+        if account is not None:
+            click.echo(f"{account.name} selected!")
+            return account
+        else:
+            click.echo("Missing an account to send disputes. Running alerts only!")
+            return None
+
+    accounts[0].unlock()
+    return accounts[0]
+
+
 def get_logger(name: str) -> logging.Logger:
     """DVM logger
 
